#define _WINSOCKAPI_
//---------------------------------------------------------------------------
#include <vcl\vcl.h>
#include <vcl\registry.hpp>
#include <vcl\clipbrd.hpp>
#pragma hdrstop

#include "MainMDI.h"
#include "TextWins.h"
#include "NoteWins.h"
#include "FileWin.h"
#include "MsgWins.h"
#include "MsgInWin.h"
#include "RptWin.h"
#include "SmtpDlgs.h"
#include "RemailerDlgs.h"
#include "FirstDocsDlgs.h"
#include "SaveDlg.h"
#include "QueueDlgs.h"
#include "OldOpenDlg.h"
#include "MailboxDlgs.h"
#include "MailboxWins.h"
#include "MixKeyWins.h"
#include "MixProbWins.h"
#include "CpkKeyWins.h"
#include "PgpKeyWins.h"
#include "PackDlg.h"
#include "AboutDlg.h"
#include "Fmxutils.h"
#include "AddressDlgs.h"
#include "AddrEditDlgs.h"
#include "HeadMan.h"
#include "Global.h"
#include "PgpPassEx.h"
#include "Wiper.h"
#include "Version.h"
#include "GetStrtDlg.h"
#include "mixmaster.h"
#include "mixmastdlg.h"
#include "MixInstDlg.h"
#include "PreferDlg.h"
#include "ClipReply.h"
#include "NewNameDlgs.h"
#include "pop3mail.h"
#include "pgpmail.h"
#include "helpdlgs.h"
#include "helpindexdlgs.h"
#include "importDlg.h"
#include "deldocsdlg.h"
#include "UPDateDlg.h"
#include "pgpfiles.h"
#include "PgpKeyWins.h"
#include "pgpsdk.h"
#include "pool.h"
#include "welcomedlgs.h"
#include "view.h"
//---------------------------------------------------------------------------
#pragma link "explbtn"
#pragma resource "*.dfm"
TMainForm *MainForm;
TDataBuf *MasterPassword;
const bool sorted = 1;
//---------------------------------------------------------------------------
void __fastcall TMainForm::OnActiveFormChange(TObject *Sender)
{
   if (!Application->Terminated)
   {
      MainForm->SetKeyringMenu();
      MainForm->SetEditMenu();
   }
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::ShowHint(TObject *Sender)
{
   StatusBar->Panels->Items[0]->Text = Application->Hint;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::OnAppActivate(TObject *Sender)
{
   TPgpKeyWin *keywin;
   if (HasPgp() && HasSdk() && PGPqdkCheckPGP6NeedsUpdate(this,NULL))
   {
      for (int i=0; i<MDIChildCount; i++)
      {
         if ((keywin=dynamic_cast<TPgpKeyWin*>(MDIChildren[i]))!=NULL)
         {
            if (MsgBoxHwnd(keywin->Handle,
               s("Your regular PGP program has just updated keyrings on disk. Would you\n")+
               s("like to refresh your in-memory keyrings?"),
               s("Keyring Change"),
               MB_OKCANCEL)==IDOK)
            {
               keywin->BringToFront();
               keywin->ClearTreeView(); Sleep(100);
               keywin->RefreshBtn->Click();
               break;
            }
         }
      }
   }
}
//---------------------------------------------------------------------------
__fastcall TMainForm::TMainForm(TComponent* Owner)
  : TForm(Owner)
{
   AnsiString pswd;
   // lets make sure we're in home dir
   if (GetCurrentDir()!=AppDir) SetCurrentDir(AppDir);
   // nullify the various libraries used
   hMixLib = NULL;
   hSSLLib = NULL;
   hPgpLib = NULL;
   hPopLib = NULL;
   hPgpSdk = NULL;

   GetThatMix = false;
   MasterPassword = NULL;
   Caption = AppName+" "+ReleaseVer;
   Application->Title = AppName.Trim();
   Screen->OnActiveFormChange = OnActiveFormChange;
   if (!CheckAppiniExists()) ExitProcess(0);
   if (ParamStr(1)=="-mix") RunMixmasterInstall(ParamStr(2));

   // load mix and crypto libs
   bool shutdown=false; // try to load libeay32.dll -- unknown problem doing it later
   hSSLLib = LoadLibrary(AnsiString(MixDir()+"libeay32.dll").c_str());
   hMixLib = CheckMixmasterInstall(shutdown);
   if (shutdown)
   {
      SafeStoreMixRandBin();
      Application->ShowMainForm = false;
      PostMessage(this->Handle,WM_QUIT,0,NULL);
      return;
   }
   // load pgplib -- load pgpsdk is accomplished in timer1timer
   if ((hPgpLib=CheckQSPgpInstall())!=NULL &&
      !PgpInit(hMixLib,hSSLLib,this,Application->Handle))
   {
      MsgBox("Failed to initialize pgp library","Dll Error",MB_OK);
      FreeLibrary(hPgpLib);
      hPgpLib = NULL;
   }
   // load pop lib
   hPopLib = CheckQSPop3Install();

   MasterPassword = new TDataBuf();
   if (!IsEncrypted() && (!FileExists(AppDat) || GetFileSize(AppDat)==0))
   {
      if (FileExists(AppPgp))
      {
         if (!HasMixmaster())
         {
            MsgBoxHwnd(NULL,
               "Mixmaster is not available.\nCannot decrypt datafile!\n\nQuitting",
                AppName,
                MB_OK
            );
            delete MasterPassword;
            ExitProcess(0);
         }
         else
         if (PGPgetDecryptionPassphrase(pswd,NULL))
         {
            MasterPassword->SetData(pswd.c_str(),pswd.Length());
            for (int i=1; i<=pswd.Length(); i++) pswd[i] = ' '; pswd = "";
            if (!UnSecureData(MasterPassword))
            {
               MsgBoxHwnd(NULL,
                 "Unable to open datafile: "+ExtractFileName(AppPgp)+"  Quitting!",
                  AppName,
                  MB_OK
               );
               delete MasterPassword;
               ExitProcess(0);
            }
         }
         else
         {
            delete MasterPassword;
            ExitProcess(0);
         }
      }
      else  // create new dat file copying qs.ini
      {
         if (FileExists(AppIni))
         {
            TMemoryStream *ms = new TMemoryStream();
            ms->LoadFromFile(AppIni);
            ms->SaveToFile(AppDat);
            delete ms;
         }
      }
   }
   DatTime = FileExists(AppDat) ? FileDateTime(AppDat):(TDateTime)0;

   // initialize window position
   TWindowState winstate=wsNormal;
   TRegistry *reg = new TRegistry();
   if (reg->OpenKey(QuickKey,false))
   {
      winpos = reg->ReadString("WindowPos");
      winstate = (TWindowState)reg->ReadInteger("WindowState");
      if (reg->ValueExists("Update")) // <--new release: first run
      {
         reg->DeleteValue("Update");
      // add code for first run purposes (on update, not new install)
      // this adds new release number to bugreport
      }
   }
   delete reg;

   // initialize window position
   int h = Screen->Height>480 ? 500:400;
   int w = Screen->Width>640 ? 750:600;
   int x = (Screen->Width-w)/2;
   int y = (Screen->Height-h)/2;
   if (winpos.IsEmpty()) winpos=s(x)+","+s(y)+","+s(w)+","+s(h);
   Left   = GetParam(0,winpos,",").ToIntDef(x);
   Top    = GetParam(1,winpos,",").ToIntDef(y);
   Width  = GetParam(2,winpos,",").ToIntDef(w);
   Height = GetParam(3,winpos,",").ToIntDef(h);
   // adjust position to screen if something is out of wak
   if (Height > Screen->Height) Height = Screen->Height;
   if (Width > Screen->Width) Width = Screen->Width;
   if (Left+Width > Screen->Width) Left = Screen->Width-Width;
   if (Top+Height > Screen->Height-28) Top = Screen->Height-28-Height;
   if (Left < -1) Left = -1;
   if (Top < -1) Top = -1;
   WindowState = winstate;

   WipeRand = new TList();
   MixmasterPoolClear();

   Color = (TColor)GetAppIni(WorkSpaceColor,"").ToIntDef(clTeal);
   OptionsBtn->Visible = FileExists(ChangeFileExt(AppExeName,".dsk"));

   LoadMailboxMenu();
   SetMenus();

   if (!CYBERPUNKENABLED) SetNoCyberpunk();
   if (!RegKeyExists(MixKey) || !hMixLib)
      SetNoMixmaster();

   ProbQueueBitmap = OpenProbsBtn->Bitmap;
   //ImageList1->GetBitmap(0,OpenProbsBtn->DisabledBitmap);
   OpenQueueBtn->Enabled = AppDataExists(MailQueue);
   OpenProbsBtn->Enabled = AppDataExists(ProblemQueue);
   //EnableProbQueueButton(AppDataExists(ProblemQueue));

   StatusBar->Panels->Items[0]->Text = "";
   StatusBar->Panels->Items[1]->Text = "";
   KeyringMenu->Visible = true;
   Timer1->Enabled = true;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Timer1Timer(TObject *Sender)
{
   Timer1->Enabled = false;
   AppProcessMessages();
   if (GetThatMix)
   {
      // open the welcome dialog with current version data
      AppProcessMessages(); Sleep(500);
      if (OpenWelcomeDialog(this))
      {
         bool cancel;
         AppProcessMessages(); Sleep(200);
         RunProgramOnExit = RunUpdateExpress(this,cancel);
         if (cancel) Show(); else Close();
         return;
      }
   }
   // load initialize pgpsdk
   if (HasPgp() && (hPgpSdk=CheckQSPgpSdkInstall())!=NULL &&
      !PGPqdkInit(this->Handle,Application->Handle))
   {
      MsgBox("Failed to initialize pgpsdk library","Dll Error",MB_OK);
      FreeLibrary(hPgpSdk);
      hPgpSdk = NULL;
   }

   AnsiString pubring=PgpPublicRing();
   PgpTime = HasSdk() && FileExists(pubring) ? FileDateTime(pubring):(TDateTime)0;

   OpenDialog->Filter = "All files (*.*)|*.*";
   OpenDialog->InitialDir = AppDir;

   //KeyringMenu->Visible = false;
   UpdateStatsBox->Checked = GetAppIni(IniUpdateOnSend,"1").ToIntDef(0);
   ToolsUpdateonSendItem->Checked = UpdateStatsBox->Checked;
   OpenDialog->InitialDir = AppDir;
   SaveDialog->InitialDir = AppDir;

   Application->OnHint = &ShowHint;
   Application->OnActivate = &OnAppActivate;

   HeadMan = new THeaderManager();
   ProcessLog = new TStringList();
   ErrorLog = new TStringList();

   if (HasPgp())
   {
      if (PgpPublicRing().IsEmpty() || PgpSecretRing().IsEmpty())
      {
         MsgBox(
            s("I see you have the QuickSilver PGP plugin installed, but QS is\n")+
            s("unable to locate your pgp keyrings. I'll open the Options Dialog\n")+
            s("so you can locate these yourself. Also, if you have PGP 6, please\n")+
            s("look over the information regarding importing these keys to QS."),
            s("QuickSilver PGP"),
             MB_OK
         );
         PutAppIni("pgp6info","1");
         GetPreferences("pgp");
         return;
      }
      if (HasSdk() && GetAppIni("pgp6info","0")=="0")
      {
         MsgBoxHwnd(Handle,
            s("I see you have PGP 6 installed. Previous to this update the method for importing\n")+
            s("v6 keys to QS involved first exporting the keys to a disk file then importing the disk\n")+
            s("file into QS. I have eliminated the need to first export v6 keys to disk.\n\n")+
            s("NOW--QS can import v6 keys directly and automatically from the v6 keyrings. I'll\n")+
            s("open the Options Dialog so that you can briefly look it over before going on. Take\n")+
            s("a look at the help there."),
            s("Modified PGP v6 Keyring Import Feature"),
            MB_OK
         );
         PutAppIni("pgp6info","1");
         GetPreferences("pgp");
         return;
      }
      PGPqdkCheckPGP6NeedsUpdate(this,NULL);
   }
   if (FileExists(mprobtxt))
   {
      TStringList *tmp = new TStringList();
      tmp->LoadFromFile(mprobtxt); // don't open if no contents
      if (tmp->Count) MixProbWin = new TMixProbWin(this,true);
      else DeleteFile(s(mprobtxt).c_str());
      delete tmp;
   }
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FormCloseQuery(TObject *Sender, bool &CanClose)
{
   // save window state information
   TRegistry *reg = new TRegistry();
   if (reg->OpenKey(QuickKey,false))
   {
      if (!FileExists(AppDir+"QS.dsk")) reg->WriteString("AppDir",AppDir);
      reg->WriteInteger("WindowState",(int)WindowState);
      if (WindowState != wsMaximized)
         winpos = s(Left)+","+s(Top)+","+s(Width)+","+s(Height);
      reg->WriteString("WindowPos",winpos);
   }
   delete reg;

   if (IsEncrypted())
   {
      if (FileDateTime(AppDat)!=DatTime)
      {
         if (!SecureData(MasterPassword))
            MsgBox("Encrypting Error: Unable to secure database!","Secure Mode",MB_OK);
         else
         {
            if (wipefile(AppDat)==WIPE_OK) // Secure Mode wipe database
               DeleteFile(AppDat.c_str());
            else
               MsgBox(
                  s("Your QuickSilver database has been successfully encrypted,\n")+
                  s("but I was unable to wipe the unencrypted database 'QS.dbc'         \n")+
                  s("from the from your hard drive.\n\n")+
                  s("Your database is not secure!"),
                  s("Secure Mode"),
                  MB_OK|MB_ICONHAND
               );
         }
      }
      else
      {
         if (wipefile(AppDat)==WIPE_OK) // Secure Mode wipe database
            DeleteFile(AppDat.c_str());
         else
            MsgBox(
               s("Your QuickSilver database has been successfully encrypted,\n")+
               s("but I was unable to wipe the unencrypted database 'QS.dbc'         \n")+
               s("from the from your hard drive.\n\n")+
               s("Your database is not secure!"),
               s("Secure Mode"),
               MB_OK|MB_ICONHAND
            );
      }
   }
   delete MasterPassword;

   /*if (HasSdk() && PGP6AUTOUPDATE &&
       FileDateTime(PgpPublicRing())!=PgpTime && FileDateTime(PGPqdkPubringpkr())<PgpTime)
   {
      if (MsgBox(
         s("Your Personal PGP Keyrings are modified. Would you like to\n")+
         s("post the changes to your PGP6 ?"),
         s("Confirm"),
         MB_YESNO)==ID_YES)
      {
         PGPqdkSaveQSKeysToPGP6(Handle);
      }
   }*/

   SafeStoreMixRandBin();
   if (OpenProbsBtn->Bitmap==OpenQueueBtn->Bitmap)
       OpenProbsBtn->Bitmap = ProbQueueBitmap;
}
//---------------------------------------------------------------------------
__fastcall TMainForm::~TMainForm()
{
   MixmasterPoolClear();
   delete WipeRand;
   PGPqdkCleanup();
   if (hMixLib) FreeLibrary(hMixLib);
   if (hSSLLib) FreeLibrary(hSSLLib);
   if (hPgpLib) FreeLibrary(hPgpLib);
   if (hPgpSdk) FreeLibrary(hPgpSdk);
   if (hPopLib) FreeLibrary(hPopLib);
   if (!RunProgramOnExit.IsEmpty())
   {
      Sleep(200);
      Exec95(RunProgramOnExit);
   }
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::EnableProbQueueButton(bool enable)
{
   // this routine fixes the off color problem with bitmat for disabled
   if (enable)
   {
      OpenProbsBtn->Enabled = true;
//      OpenProbsBtn->Bitmap = ProbQueueBitmap;
//sm("yes");
   }
   else
   {
//      OpenProbsBtn->Bitmap = OpenQueueBtn->Bitmap;
      OpenProbsBtn->Enabled = false;
//sm("no");
   }
   AppProcessMessages();
}
//---------------------------------------------------------------------------
bool __fastcall TMainForm::SecureData(TDataBuf *password)
{
   if (PgpEncryptQSDBC(password)==mrOk && FileExists(AppPgp)) return true;
   return false;
}
//---------------------------------------------------------------------------
bool __fastcall TMainForm::UnSecureData(TDataBuf *password)
{
   if (!FileExists(AppPgp)) return true;
   if (PgpDecryptQSDBC(password)==mrOk && FileExists(AppDat)) return true;
   password->Clear();
   return false;
}
//---------------------------------------------------------------------------
bool __fastcall TMainForm::IsEncrypted()
{
   return (MasterPassword!=NULL && !MasterPassword->IsEmpty());
}
//---------------------------------------------------------------------------
bool __fastcall TMainForm::CheckRegkeyExists()
{
   bool ok = true;
   if (!RegKeyExists(QuickKey))
   {
      MsgBoxHwnd(NULL,
         AppName+"'s registry entries are missing. "+
        "You must install, or re-install,\nthe program! "+
        "If you currently have an application data file, "+
        "'"+ExtractFileName(AppDat)+"', it will\nbe preserved through a re-install.",
         AppName+" Error",
         MB_OK
      );
      ok = false;
   }
   return ok;
}
//---------------------------------------------------------------------------
bool __fastcall TMainForm::CheckAppiniExists()
{
   if (!FileExists(AppIni))
   {
      AnsiString appini = ExtractFileName(AppIni);
      MsgBoxHwnd(NULL,
        AppName+"'s '"+appini+"' file is missing. You will need to replace\n"+
        "it from the distribution zip file you installed the program from.\n\n"+
        "Clue: Though the file you downloaded had a '.exe'\n"+
        "extension, it was, in fact, a self-extracting ZIP file. To\n"+
        "extract just the '"+appini+"' file, open the distribution zip file with\n"+
        "a zip program, such as Winzip, and extract just '"+appini+"'.\n"+
        "Then place that file in "+AppName+"'s home directory.\n\n"+
        "Alternatively, you can simply re-install "+AppName+" to the\n"+
        "same directory as before. That will solve the problem and\n"+
        "your current data file will be preserved.",
        AppName+" Ini Error",
        MB_OK
      );
      return false;
   }
   return true;
}
//---------------------------------------------------------------------------
HINSTANCE __fastcall TMainForm::CheckQSPop3Install()
{
   HINSTANCE hpoplib = LoadLibrary(s(AppDir+QSpop3Name).c_str());
   if (!hpoplib) SetNoPop3();
   return hpoplib;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::SetNoPop3()
{
   CheckMailBtn->Visible = false;
   for (int i=0; i<Panel1->ControlCount; i++)
      if (Panel1->Controls[i]->Left > CheckMailBtn->Left)
          Panel1->Controls[i]->Left -= CheckMailBtn->Width;
   InBoxBtn->Visible = false;
   for (int i=0; i<Panel1->ControlCount; i++)
      if (Panel1->Controls[i]->Left > InBoxBtn->Left)
          Panel1->Controls[i]->Left -= InBoxBtn->Width;
   FileCheckMailItem->Visible = false;
   ToolsPopAccountsItem->Visible = false;
   //FileSMTPLogItem->Visible = true;
   FileSessionLogsItem->Visible = true;
   SessionLogsPOP3Item->Visible = false;
   ViewLogsPop3->Visible = false;
   //MailboxInboxItem->Visible = false;
   //ClipboardReceiveItem->Visible = false;
   //ClipboardN1Item->Visible = false;
   HelpQuickPop3Item->Visible = false;
   HelpN12Item->Visible = false;
}
//---------------------------------------------------------------------------
HINSTANCE __fastcall TMainForm::CheckQSPgpInstall()
{
   HINSTANCE hpgplib=NULL;
   if (hSSLLib && hMixLib)
      hpgplib = LoadLibrary(AnsiString(AppDir+QSpgpName).c_str());
   if (!hpgplib) SetNoPgp();
   PgpMenu->Visible = false; // <-temporary til pgp more complete
   return hpgplib;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::SetNoCyberpunk()
{
   // removes the cyberpunk tool from QS main toolbar
   if (HasPgp() && CpkKeysBtn->Visible)
   {
      CpkKeysBtn->Visible = false;
      for (int i=0; i<Panel1->ControlCount; i++)
      {
         if (Panel1->Controls[i]->Left>CpkKeysBtn->Left)
             Panel1->Controls[i]->Left -= CpkKeysBtn->Width;
      }
      ToolsCpunkManItem->Visible = false;
   }
}
void __fastcall TMainForm::SetCyberpunk()
{
   // removes the cyberpunk tool from QS main toolbar
   if (HasPgp() && !CpkKeysBtn->Visible)
   {
      CpkKeysBtn->Visible = true;
      for (int i=0; i<Panel1->ControlCount; i++)
      {
         if (Panel1->Controls[i]==CpkKeysBtn) continue;
         if (Panel1->Controls[i]->Left>=CpkKeysBtn->Left)
             Panel1->Controls[i]->Left += CpkKeysBtn->Width;
      }
      ToolsCpunkManItem->Visible = true;
   }
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::SetNoPgp()
{
   PgpMenu->Visible = false;
   HelpQuickPgpItem->Visible = false;
   //
   //ToolsPgpKeyItem->Visible = false;
   //ToolsMixManItem->Visible = false;
   //ToolsCpunkManItem->Visible = false;
   ToolsKeyringsItem->Visible = false;
   ToolsMixKeyringItem->Visible = true;
   ToolsMixKeyringItem->Enabled = true;
   //
   ViewPubringAscItem->Visible = false;
   ViewRListTxtItem->Visible = false;
   CpkKeysBtn->Visible = false;
   PgpKeysBtn->Visible = false;
   N2->Visible = false;
   for (int i=0; i<Panel1->ControlCount; i++)
   {
      if (Panel1->Controls[i]->Left>MixKeysBtn->Left)
          Panel1->Controls[i]->Left -= CpkKeysBtn->Width+PgpKeysBtn->Width;
   }
}
//---------------------------------------------------------------------------
HINSTANCE __fastcall TMainForm::CheckQSPgpSdkInstall()
{
   HINSTANCE hpgpsdk=NULL;
   AnsiString pgp6=AppDir+QSpgp6Name;
   if (FileExists(pgp6)) hpgpsdk = LoadLibrary(pgp6.c_str());
   if (!hpgpsdk) SetNoPgpSdk();
   return hpgpsdk;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::SetNoPgpSdk()
{
}
//---------------------------------------------------------------------------
HINSTANCE __fastcall TMainForm::CheckMixmasterInstall(bool &shutdown)
{
   HINSTANCE HMixLib;//,HSSLLib=NULL;
   bool neededmix=!RegKeyExists(MixKey);
   AnsiString mixdir,appdir=ExtractFilePath(AppExeName);

   if (neededmix && MixZipExists() && RunMixmasterInstall()==mrOk)
   {
      //shutdown = true;
      //return NULL;
   }

   if (!RegKeyExists(MixKey))
   {
      GetThatMix = true;
      return NULL;
   }

   TRegistry *reg = new TRegistry();
   if (reg->OpenKey(MixKey,false)) mixdir = reg->ReadString("MixDir");
   delete reg;
   if (mixdir.IsEmpty() || !DirectoryExists(mixdir))
   {
      MsgBoxHwnd(NULL,
         s("I have found a problem! The required Mixmaster\n")+
         s("key: HKEY_CURRENT_USER/Software/Mixmaster\n")+
         s("exists in the windows registry, but the subkey MixDir\n")+
         s("is missing or assigned an invalid value. If you know how\n")+
         s("take a look at the registry using regedit.exe.\n\n")+
         s("This key should tell me where I can find mixmaster so I\n")+
         s("can load it into memory. I cannot initialize mix and so,\n")+
         s("cannot provide mixmaster functionality. This is a problem\n")+
         s("that needs to be corrected. Best option is deleting the\n")+
         s("mix key named above, and allowing QS to reinstall mix.\n"),
         AppName,
         MB_OK
      );
      return NULL;
   }

   if (!FileExists(AddBackslash(mixdir)+"mixlib.dll"))
   {
      MsgBoxHwnd(NULL,
         s("I have found a problem! Mixmaster appears to be\n")+
         s("installed. I have found the required Mixmaster key\n")+
         s("is present in the windows registry, and the MixDir\n")+
         s("subkey is assigned an existing directory name, BUT\n")+
         s("I cannot find the required file mixlib.dll in that\n")+
         s("dir. It should be and needs to be there. You'll need\n")+
         s("to look into the problem.\n\n")+
         s("Without mixlib.dll, I cannot provide mix functions.\n"),
         AppName,
         MB_OK
      );
      return NULL;
   }

   if (!FileExists(AddBackslash(mixdir)+"libeay32.dll"))
   {
      MsgBoxHwnd(NULL,
         s("I have found a problem! Mixmaster appears to be\n")+
         s("installed. I have found the required Mixmaster key\n")+
         s("is present in the windows registry, and the MixDir\n")+
         s("subkey is assigned an existing directory name, BUT\n")+
         s("I cannot find the required file libeay32.dll in that\n")+
         s("dir. It should be and needs to be there. You'll need\n")+
         s("to look into the problem.\n\n")+
         s("Without libeay32.dll, I cannot provide mix\n")+
         s("functions.\n"),
         AppName,
         MB_OK
      );
      return NULL;
   }

   SetCurrentDir(mixdir);
   if (!FileExists(MixCfg))
   {
      TStringList *cfg = new TStringList();
      GetAppData(cfg,MixValues+PS+MixClientCfg); // get one stored in appdat
      if (!cfg->Count) cfg->Add(DefMixCfg);      // use hardcode version
      cfg->SaveToFile(MixCfg);
      delete cfg;
   }

   if ((HMixLib=LoadLibrary("mixlib.dll"))==NULL)
   {
      MsgBoxHwnd(NULL,
         s("Unable to locate mixlib.dll.\n")+
         s("System Error: "+GetLastErrorStr()+"\n\n")+
         s("Remailer chaining and secure mode will not be available.\n\n")+
         s("Mixmaster DLL's must be in either a directory listed in\n")+
         s("your computer's PATH statement OR in "+AppName+"'s home\n")+
         s("directory, (need not be listed in PATH)"),
         AppName,
         MB_OK
      );
   }
   SetCurrentDir(appdir);
   return HMixLib;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::SetNoMixmaster()
{
   FileSecureModeItem->Visible = false;
   ViewMenu->Visible = false;
   //ViewErrLogItem->Enabled = false;
   //ViewMixcfgItem->Enabled = false;
   //ViewPubringMixItem->Enabled = false;
   //ViewMListTxtItem->Enabled = false;
   //ViewType2ListItem->Enabled = false;
   //ViewPubringAscItem->Enabled = false;
   //ViewRListTxtItem->Enabled = false;
   //ViewClearItem->Enabled = false;
   //ToolsRemailersItem->Enabled = false;
   //ToolsUpdateonSendItem->Enabled = false;
   //ToolsUpdateonSendItem->Checked = false;
   ToolsRemailersItem->Visible = false;
   ToolsUpdateonSendItem->Visible = false;
   // hide keyring submenu; make mixkeyring item visible
   ToolsKeyringsItem->Visible = false;
   ToolsMixKeyringItem->Visible = false;
   OpenMixDirectoryItem->Visible = false;
   N3->Visible = false;
   //ToolsMixKeyringItem->Enabled = false;
   RemailerBtn->Enabled = false;
   RemailerBtn->Visible = false;
   UpdateStatsBox->Visible = false;
   //UpdateStatsBox->Enabled = false;
   //UpdateStatsBox->Checked = false;
   ToolsMixManItem->Enabled = false;
   MixKeysBtn->Visible = false;
   CpkKeysBtn->Visible = false;
   Bevel10->Visible = false;
   Bevel12->Visible = false;
   for (int i=0; i<Panel1->ControlCount; i++)
      if (Panel1->Controls[i]->Left>OutBoxBtn->Left)
         Panel1->Controls[i]->Left -= (MixKeysBtn->Width*2)+UpdateStatsBox->Width+27;
   SessionLogsErrorCheckItem->Visible = false;
}
//---------------------------------------------------------------------------
bool __fastcall TMainForm::CheckMixrndExists()
{
   if (!RegKeyExists(MixKey)) return true;

   bool ok = true; TDataBuf *buf = new TDataBuf();
   AnsiString randpool = MixValues+PS+AppRandPool;
   // make sure mixrand.bin has been saved in ini file
   if (FileExists(MixRandBin) && !AppBinaryExists(randpool,buf,AppIni))
   {
      buf->LoadFromFile(MixRandBin);
      PutAppBinary(randpool,buf,AppIni);
   }
   // if no mixrand.bin attemp to restore from ini file OR dat file
   if (!FileExists(MixRandBin) &&
      (AppBinaryExists(randpool,buf,AppIni) || AppBinaryExists(randpool,buf)))
   {
      buf->SaveToFile(MixRandBin);
   }
   delete buf;
   // no mixrand.bin and no backup...
   if (!FileExists(MixRandBin) && !AppBinaryExists(randpool,NULL,AppIni))
   {
      MsgBoxHwnd(NULL,
         "Mixmaster's file 'mixrand.bin' is missing. You'll need to reinstall mixmaster!. Make sure the latest version of Mix's zip file is in the "+AppName+" home directory, and restart "+AppName+".",
          AppName,
          MB_OK
      );
      // remove regkey so that program will install mixmaster, next startup
      // if mixmaster zip is quicksilver dir. leave mix files.
      if (RegKeyExists(MixKey))
      {
         TRegistry *reg = new TRegistry();
         reg->DeleteKey(MixKey);
         delete reg;
      }
      ok = false;
   }
   return ok;
}
//---------------------------------------------------------------------------
bool __fastcall TMainForm::MixZipExists()
{
   return GetFilenames(NULL,AppDir+"mix29*.zip") || GetFilenames(NULL,AppDir+"mix3*.zip");
}
//---------------------------------------------------------------------------
//bool __fastcall TMainForm::HasMixmaster()
//{
//   return RegKeyExists(MixKey) && hMixLib!=NULL;
//}
//---------------------------------------------------------------------------
void __fastcall TMainForm::ShowInitValues()
{
  // Show the initialization values
  TStringList *init1 = new TStringList();
  TStringList *init2 = new TStringList();
  GetAppData(init1,AppIniValues);
  init1->Insert(0,"[Internal]");
  init2->LoadFromFile(AppIni);
  init1->Add("");
  init1->AddStrings(init2);
  ShowMessage(init1->Text);
  delete init2;
  delete init1;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FormDestroy(TObject *Sender)
{
   AnsiString unzipApp = AppDir+UnzipApp;
   if (FileExists(unzipApp)) DeleteFile(unzipApp.c_str());
   if (hMixLib) FreeLibrary(hMixLib);
   delete HeadMan;
   delete ErrorLog;
   delete ProcessLog;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::WMQueueChange(TMessage& Msg)
{
   OpenQueueBtn->Enabled = AppDataExists(MailQueue);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::WMProblemChange(TMessage& Msg)
{
   //OpenProbsBtn->Enabled = AppDataExists(ProblemQueue);
   EnableProbQueueButton(AppDataExists(ProblemQueue));
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::WMUpdateChecked(TMessage& Msg)
{
   Msg.Result=0;
   if (ToolsUpdateonSendItem->Checked)
   {
      AnsiString laststr;
      double minbetween,last=0,curr=(double)Now();
      switch (Msg.WParam)
      {
         case 0:
            minbetween=GetAppIni(StatsMinUpdates,"").ToIntDef(0);
            laststr = GetAppIni("LastStatsTime","");
            if (!laststr.IsEmpty()) last = laststr.ToDouble();
            Msg.Result = last==0 || curr-last>=minbetween/1440;
            //tmp=minbetween/1440; sm(AnsiString(curr-last)+"\n"+AnsiString(tmp));
            break;
         case 1:
            PutAppIni("LastStatsTime",AnsiString((double)curr));
            break;
      }
   }
}
//---------------------------------------------------------------------------
void  __fastcall TMainForm::WMUpdateEnable(TMessage& Msg)
{
   ToolsUpdateonSendItem->Enabled = (bool)Msg.WParam;
   UpdateStatsBox->Enabled = (bool)Msg.WParam;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::WMCheckUpdate(TMessage& Msg)
{
   if (ToolsUpdateonSendItem->Checked!=(bool)Msg.WParam)
       ToolsUpdateonSendItem->Click();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::UpdateStatsBoxClick(TObject *Sender)
{
   if (ToolsUpdateonSendItem->Checked != UpdateStatsBox->Checked)
       ToolsUpdateonSendItem->Click();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::WMInsertFilename(TMessage& Msg)
{
   TTextWin *tw = dynamic_cast<TTextWin*>(ActiveMDIChild);
   if (tw && OpenDialog->Execute())
       tw->Editor->SelText = OpenDialog->FileName;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::WMSetButtons(TMessage& Msg)
{
   SetMenus(Msg.WParam);
//   bool sel = Msg.LParam > 0;
   if (Msg.WParam)//enable
   {
//      EditUndoItem->Enabled = true;
//      EditCutItem->Enabled = sel;
//      EditCopyItem->Enabled = sel;
//      EditPasteItem->Enabled = Clipboard()->HasFormat(CF_TEXT);
//      EditSelectAllItem->Enabled = true;

//      CutBtn->Enabled = sel;
//      CopyBtn->Enabled = sel;
//      PasteBtn->Enabled = Clipboard()->HasFormat(CF_TEXT);
//      SetMenus();
   }
   else
   {
//      EditUndoItem->Enabled = false;
//      EditCutItem->Enabled = false;
//      EditCopyItem->Enabled = false;
//      EditPasteItem->Enabled = false;
//      EditSelectAllItem->Enabled = false;

//      PasteBtn->Enabled = false;
//      CopyBtn->Enabled = false;
//      CutBtn->Enabled = false;
//      SetMenus(false);
   }
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::WMGetErrorLog(TMessage& Msg)
{
   AnsiString entry,MixLog = MixDir()+"errlog";
   if (!FileExists(MixLog)) return;
   TStringList *newEntries = new TStringList();
   switch (Msg.WParam)
   {
      case 0: ErrorLog->LoadFromFile(MixLog); break;     // load errorlog on 0
      case 1:     // success, load errorlog into newentries...
      {           // any new entries since load on 0
         newEntries->LoadFromFile(MixLog);   // show the lines to the user
         for (int i=ErrorLog->Count; i<newEntries->Count; i++)
         {
            entry = newEntries->Strings[i];
            SmtpDlg->Display->Lines->Add("Mixmaster reports: "+entry);
         }
         ErrorLog->Text = newEntries->Text;
         break;
      }
      case 2:
      {
         newEntries->LoadFromFile(MixLog);
         for (int i=ErrorLog->Count; i<newEntries->Count; i++)
         {
            entry = newEntries->Strings[i];
            SmtpDlg->Display->Lines->Add("Mix Error: "+entry);
         }
         ErrorLog->Text = newEntries->Text;
         break;
      }
   }
   delete newEntries;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::WMDisplayXY(TMessage& Msg)
{
   AnsiString display("            ");
   if (Msg.LParam>=0)
      display = AnsiString((int)Msg.LParam)+"/"+AnsiString((int)Msg.WParam);
   StatusBar->Panels->Items[1]->Text = display;
}
//---------------------------------------------------------------------------
/*void __fastcall TMainForm::WMOpenHelp(TMessage& Msg)
{
   AnsiString sec,tit,params((char*)Msg.LParam);
   sec = GetParam(0,params,";");
   tit = GetParam(1,params,";");
   OpenHelpModal(sec,tit);
}*/
//---------------------------------------------------------------------------
void __fastcall TMainForm::WMOpenProblems(TMessage& Msg)
{
   FileProblemsItem->Click();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::WMOpenQueue(TMessage& Msg)
{
   FileOpenQueueItem->Click();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::WMLoadMailboxMenu(TMessage& Msg)
{
   LoadMailboxMenu();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::WMOpenAddresses(TMessage& Msg)
{
   ToolsAddressBookItem->Click();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::WMStorePoplog(TMessage& Msg)
{
   TStringList *log = new TStringList();
   log->SetText((char*)Msg.LParam);
   SavePOP3Log((WORD)Msg.WParam,log);
   delete log;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::WMOpenInbox(TMessage& Msg)
{
   MailboxWin = OpenMailbox("inbox");
   MailboxWin->MailGrid->RefreshIndex();
   MailboxWin->MailGrid->ClearSelected();
   MailboxWin->GoNewMessages();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::WMSetWinCursor(TMessage& Msg)
{
   Cursor = (TCursor)Msg.WParam;
}
//---------------------------------------------------------------------------
// not used--done in pop3 now
void __fastcall TMainForm::WMIncomingMail(TMessage& Msg)
{
   TStringList *msg = new TStringList();
   msg->SetText((char*)Msg.LParam);
   if (msg->Count)
      PutAppData(UserMailboxes+PS+"inbox"+PS+HeadMan->InIndexString(msg),msg);
   Msg.Result = 1;
   delete msg;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::WMPopAccounts(TMessage& Msg)
{
   TStringList *poppers = new TStringList();
   switch (Msg.WParam)
   {
      case 0:
         GetAppData(poppers,PopAccounts);
         Msg.Result = (long)poppers->Text.c_str();
         break;
      case 1:
         poppers->SetText((char*)Msg.LParam);
         PutAppData(PopAccounts,poppers);
         break;
   }
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::WMDatRequest(TMessage& Msg)
{
   static AnsiString datstr; // for returning data in getdata
   TStringList *data = new TStringList();
   data->SetText((char*)Msg.LParam);
   if (data->Count)
   {
      AnsiString path = data->Strings[0]; data->Delete(0); //path is passes with datalist
      switch (Msg.WParam)
      {
         case GetDatData:
            GetAppData(data,path); datstr = data->Text;
            Msg.Result = (long)datstr.c_str();
//sm(data->Text);
            break;
         case PutDatData:
            PutAppData(path,data);
//sm(data->Text);
            break;
      }
   }
   if (Msg.WParam!=GetDatData) datstr = "";
   delete data;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::WMShowStatus(TMessage& Msg)
{
   AnsiString msg((char*)Msg.LParam);
   if (Msg.WParam==0) DisplayStatus(msg,"");
   else DisplayStatus("",msg);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::WMSaveQuit(TMessage& Msg)
{
// not working
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::WMSecureData(TMessage& Msg)
{
// not working
   switch (Msg.WParam)
   {
      case 1: if (IsEncrypted()) SecureData(MasterPassword); break;
   }
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::WMHelpWinPos(TMessage& Msg)
{
   AnsiString pos;
   char winpos[32];
   switch (Msg.WParam)
   {
      case 1:
         strcpy(winpos,GetAppIni(HelpWinPos,"").c_str());
         Msg.Result = (long)winpos;
         break;
      case 0:
         if (Msg.LParam!=NULL)
            strcpy(winpos,(char*)Msg.LParam);
         pos = winpos;
         if (!pos.IsEmpty())
            PutAppIni(HelpWinPos,pos);
         break;
   }
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::WMOpenKeyMan(TMessage& Msg)
{
   AnsiString md=MixDir(),filename((char*)Msg.LParam);
   if (FileExists(filename))
      MixKeyWin = new TMixKeyWin(this,md+MProbTxt,md+MTyp2Txt);
}
//---------------------------------------------------------------------------
/*void __fastcall TMainForm::WMEditMenu(TMessage& Msg)
{
   if (Msg.WParam==1)
   {
      EditMenu = Ed0;
      Ed0 = (TMenuItem*)(Msg.LParam);
   }
   else Ed0 = EditMenu;
}*/
//---------------------------------------------------------------------------
void __fastcall TMainForm::SafeStoreMixRandBin()
{
   // store mixrand.bin in appini
   TDataBuf *buf = new TDataBuf();
   if (FileExists(MixRandBin) && !AppBinaryExists(MixValues+PS+AppRandPool,buf,AppIni))
   {
      buf->LoadFromFile(MixRandBin);
      PutAppBinary(MixValues+PS+AppRandPool,buf,AppIni);
   }
   delete buf;
}
//---------------------------------------------------------------------------
//void __fastcall TMainForm::EditTextFile(AnsiString &FileName)
//{
//   TextFileWin = new TTextFileWin(FileName,this);
//}
//---------------------------------------------------------------------------
void __fastcall TMainForm::ViewMenuClick(TObject *Sender)
{
   bool errcheck = GetAppIni(StatsErrCheck,"").ToIntDef(1);
   AnsiString md=MixDir();
   ViewChainLogItem->Enabled   = HasMix();
   //==disable when no file exists========================
   ViewErrLogItem->Enabled     = HasMix() && FileExists(md+"errlog");
   ViewMListTxtItem->Enabled   = HasMix() && FileExists(md+MListTxt);
   ViewPubringMixItem->Enabled = HasMix() && FileExists(md+MRingTxt);
   ViewType2ListItem->Enabled  = HasMix() && FileExists(md+MTyp2Txt);

   ViewRListTxtItem->Enabled   = HasMix() && FileExists(md+CListTxt);
   ViewPubringAscItem->Enabled = HasMix() && FileExists(md+CRingTxt);
   ViewType1ListItem->Enabled  = HasMix() && FileExists(md+CTyp1Txt) && errcheck;
   // hide the cpunk items if no cpunk
   ViewRListTxtItem->Visible   = HasMix() && CYBERPUNKENABLED;
   ViewPubringAscItem->Visible = HasMix() && CYBERPUNKENABLED;
   ViewType1ListItem->Visible  = HasMix() && CYBERPUNKENABLED;
   N13->Visible                = HasMix() && CYBERPUNKENABLED;
   // hide type1.list if not error checking mode and has pgp
   ViewType1ListItem->Visible = HasPgp() && CYBERPUNKENABLED && errcheck;
   ViewClearItem->Enabled = HasMix();// && errcheck;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::ViewMixmasterFileItemClick(TObject *Sender)
{
   int pos;
   TMenuItem *item=dynamic_cast<TMenuItem*>(Sender);
   if (item)
   {
      AnsiString filename = MixDir()+item->Caption;
      while ((pos = filename.Pos("&")) > 0) filename.Delete(pos,1);
      if (!FileExists(filename)) return;
      EditTextFile(filename);
   }
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::ViewErrLogItemClick(TObject *Sender)
{
   EditTextFile(MixDir()+"errlog");
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::ToolsAddressBookItemClick(TObject *Sender)
{
   if ((MessageWin=dynamic_cast<TMessageWin*>(ActiveMDIChild))==NULL)
   {
      EditAddressesEtcOnly(this);
      return;
   }
   GetMailRecipients(MessageWin);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::ToolsRemailersItemClick(TObject *Sender)
{
   GetRemailerStats(this);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::ToolsUpdateonSendItemClick(TObject *Sender)
{
   bool checked = !ToolsUpdateonSendItem->Checked;
   ToolsUpdateonSendItem->Checked = checked;
   if (UpdateStatsBox->Checked != checked) UpdateStatsBox->Checked = checked;
   PutAppIni(IniUpdateOnSend,(checked ? "1" : "0"));
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::MailboxMailboxesItemClick(TObject *Sender)
{
   OpenMailboxes();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::MailboxItemOnClick(TObject *Sender)
{
   TMenuItem *item;
   if ((item=dynamic_cast<TMenuItem*>(Sender))==NULL) return;

   OpenMailbox(item->Caption);
   LoadMailboxMenu();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::LoadMailboxMenu()
{
   int i,c;
   AnsiString s;
   TMenuItem *newitem;
   TStringList *boxes = new TStringList(); boxes->Sorted = true;
   GetAppData(boxes,MailboxBoxmenu);
   int precount = boxes->Count;
   i=0; while (i<boxes->Count)
   {
      s = boxes->Strings[i];
      if (s.IsEmpty() || s==UserMailboxes || s=="outbox" || (HasPop3() && s=="inbox"))
         boxes->Delete(i); else i++;
   }
   // test box list against list of all current boxes, delete if not there.
   TStringList *allboxes = new TStringList();
   GetAppData(allboxes,UserMailboxes);
   i=0; while (i<boxes->Count)
   {
      if (allboxes->IndexOf(boxes->Strings[i])<0)
         boxes->Delete(i); else i++;
   }
   if (boxes->Count!=precount) PutAppData(MailboxBoxmenu,boxes);
   delete allboxes;
   c = HasPop3() ? 5 : 5; // the number of lines (above divider) on mailbox menu
   while (MailboxMenu->Count>c) MailboxMenu->Delete(c);
   for (i=0; i<boxes->Count; i++)
   {
      newitem = new TMenuItem(this);
      newitem->OnClick = MailboxItemOnClick;
      newitem->Caption = boxes->Strings[i];
      MailboxMenu->Add(newitem);
   }
   delete boxes;
}
//---------------------------------------------------------------------------
int eoh(TStrings *msg)
{
   int e=0; while (e<msg->Count && !msg->Strings[e].IsEmpty()) e++;
   return e;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::ClipboardReplyItemClick(TObject *Sender)
{
   ClipboardReply(UseDialog);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::ClipboardFollowupItemClick(TObject *Sender)
{
   ClipboardFollowup(UseDialog);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FileNewMessageItemClick(TObject *Sender)
{
   AnsiString blank;
   bool locked = false; //LockWindowUpdate(GetDesktopWindow());
   try
   {
      TStringList *msg = new TStringList();
      GetAppData(msg,DefaultMessage);
      MessageWin = new TMessageWin(msg,blank,blank,this);
      delete msg;
   }
   catch (...)
   {
      if (locked) LockWindowUpdate(0);
   }
   if (locked) LockWindowUpdate(0);
}
void __fastcall TMainForm::NewMessage()
{
   FileNewMessageItem->Click();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FileNewArticleItemClick(TObject *Sender)
{
   AnsiString blank;
   bool locked = false; //LockWindowUpdate(GetDesktopWindow());
   try
   {
      TStringList *art = new TStringList();
      GetAppData(art,DefaultArticle);
      MessageWin = new TMessageWin(art,blank,blank,this);
      MessageWin->Type = tNewArticle;
      delete art;
   }
   catch (...)
   {
      if (locked) LockWindowUpdate(0);
   }
   if (locked) LockWindowUpdate(0);
}
void __fastcall TMainForm::NewArticle()
{
   FileNewArticleItem->Click();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FileNewBugReportClick(TObject *Sender)
{
   int idx=0;
   AnsiString blank,TemplateName;
   if ((TemplateName=GetMergeMessage("Create Bug Report using...",modeMessage)).IsEmpty()) return;

   TStringList *msg = new TStringList();
   GetAppData(msg,TemplateName);
   for (int i=0; i<msg->Count; i++)
   {
      if (msg->Strings[i].LowerCase().Pos("to:")==1)
          msg->Strings[i] = "To: "+BugReportAddress;
      if (msg->Strings[i].LowerCase().Pos("subject:")==1)
          msg->Strings[i] = "Subject: Bug Report";
   }
   msg->Add(""); msg->Add("");
   for (int i=0; i<msg->Count; i++)
   {
      if (msg->Strings[i].IsEmpty())
      {
         idx=i; break;
      }
   }
   msg->Insert(++idx,"It happened when I...");
   msg->Insert(++idx,"");
   msg->Insert(++idx,"I got an error message that said...");
   msg->Insert(++idx,"");
   msg->Insert(++idx,"Version Info------------");
   msg->Insert(++idx,AppName+" "+ReleaseVer);
   if (HasPop3())
   {
      AnsiString pop3dll = AnsiString(GetPop3Version());
      if (pop3dll.IsEmpty()) pop3dll = ".9b1";
      msg->Insert(++idx,"QSpop3      "+pop3dll);
   }
   if (HasPgp())
   {
      AnsiString pgpdll = AnsiString(GetPgpLibVersion());
      msg->Insert(++idx,"QSpgp       "+pgpdll);
   }
   msg->Insert(++idx,"");
   bool locked = false; //LockWindowUpdate(GetDesktopWindow());
   try
   {
      MessageWin = new TMessageWin(msg,blank,blank,this);
      delete msg;
   }
   catch (...)
   {
      if (locked) LockWindowUpdate(0);
   }
   if (locked) LockWindowUpdate(0);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::HelpAboutItemClick(TObject *Sender)
{
   ShowAboutBox(this);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FileSecureModeItemClick(TObject *Sender)
{
   AnsiString pswd;
   bool cancel=FileSecureModeItem->Checked;
   if (PGPgetSecureModePassphrase(cancel,pswd))
   {
      MasterPassword->SetData(pswd.c_str(),pswd.Length());
      for (int i=1; i<=pswd.Length(); i++) pswd[i] = ' '; pswd = "";
      if (!MasterPassword->IsEmpty())
      {
         MsgBox("Ok, you're in Secure Mode!","QuickSilver",MB_OK);
         DatTime = 0;
//       Close();
      }
   }
   else
   {
      if (cancel)
      {
         MsgBox("Secure Mode cancelled!","QuickSilver",MB_OK);
         MasterPassword->Clear();
      }
   }
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FilePackDatabaseItemClick(TObject *Sender)
{
   PackDatabase();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FileSave(AnsiString Section)
{
   AnsiString path;
   if ((TextWin=dynamic_cast<TTextWin*>(ActiveMDIChild))!=NULL)
   {
      path = GetSaveMessagePath(Section,TextWin->Name());
      if (!path.IsEmpty())
      {
         PutAppData(path,TextWin->Editor->Lines);
         // reload note
         TextWin->Editor->Clear();
         Application->ProcessMessages(); Sleep(100);
         GetAppData(TextWin->Editor->Lines,path);
      }
   }
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FileOpen(AnsiString Section)
{
   int i,clrtxt;
   AnsiString blank="";
   TStringList *tmp = new TStringList();
   AnsiString name = OpenOpenDlg(tmp,Section);
   if (name.IsEmpty()) { delete tmp; return; }

   if (Section==ProblemQueue)
   {
      // remove 'Word-Wrap: ' header, if present
      THeaderManager *hm = new THeaderManager(tmp);
      hm->Delete(WordWrapHeader);
      delete hm;
      // get clrtxt and delete header
      TStringList *msg = new TStringList();
      if ((clrtxt=QSPoolHeaderToClearText(tmp))>0)
      {
         QSPoolGet(msg,clrtxt);
         // move any error message
         i=0; while (i<tmp->Count && tmp->Strings[i].SubString(1,1)=="#")
         {
            msg->Insert(i,tmp->Strings[i]);
            i++;
         }
         tmp->Text = msg->Text;
         delete msg;
      }
      MessageWin = new TMessageWin(tmp,blank,blank,this);
      MessageWin->DataKey = "";
      PutAppData(ProblemQueue+PS+name,NULL);
      //OpenProbsBtn->Enabled = AppDataExists(ProblemQueue);
      EnableProbQueueButton(true);//AppDataExists(ProblemQueue));
      DisplayStatus("Message removed from queue!");
      MessageWin->Editor->Modified = true;
      MessageWin->Type = tNewMessage;
      MyBeep(MB_OK);
   }
   else
   if (Section==MailQueue)
   {
      // remove 'Word-Wrap: ' header, if present
      THeaderManager *hm = new THeaderManager(tmp);
      hm->Delete(WordWrapHeader);
      delete hm;
      // get clrtxt and delete header
      if ((clrtxt=QSPoolHeaderToClearText(tmp))>0)
         QSPoolGet(tmp,clrtxt);
      MessageWin = new TMessageWin(tmp,blank,blank,this);
      MessageWin->DataKey = "";
      PutAppData(MailQueue+PS+name,NULL);
      OpenQueueBtn->Enabled = AppDataExists(MailQueue);
      DisplayStatus("Message removed from queue!");
      MessageWin->Editor->Modified = true;
      MessageWin->Type = tNewMessage;
      MyBeep(MB_OK);
   }
   else
   if (Section==UserTemplates)
   {
      MessageWin = new TMessageWin(tmp,Section,name,this);
      MessageWin->DataKey = UserTemplates+PS+name;
      MessageWin->Caption = UserTemplates+": "+name;
      MessageWin->Type = tTemplate;
   }
   else
   if (Section==UserMessages)
   {
      MessageWin = new TMessageWin(tmp,Section,name,this);
      MessageWin->DataKey = UserMessages+PS+name;
      MessageWin->Caption = UserMessages+": "+name;
      MessageWin->Type = tDraft;
   }
   else
   if (Section==UserNotes)
   {
      NoteWin = new TNoteWin(this);
      NoteWin->DataKey = UserNotes+PS+name;
      NoteWin->SetText(name,tmp);
      NoteWin->Type = tNote;
   }
   delete tmp;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::ViewLastSMTPLogItemClick(TObject *Sender)
{
   ReportWin = new TReportWin(ProcessLog,AnsiString("SMTP Process Log"),this);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FileOpenDataItemClick(TObject *Sender)
{
   FileOpen(UserData);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FileOpenQueueItemClick(TObject *Sender)
{
   OpenQueuedMail(MailQueue);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FileProblemsItemClick(TObject *Sender)
{
   OpenQueuedMail(ProblemQueue);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FileNotesItemClick(TObject *Sender)
{
   FileOpen(UserNotes);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::CheckTemplateHostFrom()
{
   TMessageWin *win = dynamic_cast<TMessageWin*>(ActiveMDIChild);
   if (!win) return;
   TStrings *lines = win->Editor->Lines;
   if (HeadMan->GetDat("From: ",lines).Trim().IsEmpty() ||
       HeadMan->GetDat("Host: ",lines).Trim().IsEmpty())
   {
      AppProcessMessages(); Sleep(100);
      if (MsgBox(
         s("This template has the 'Host:' and/or 'From:' fields\n")+
         s("empty. Would you like me to copy the info from your\n")+
         s("your 'default' new message?"),
         "Confirm",MB_YESNO)==IDYES)
      {
         AnsiString field;
         TStringList *msg = new TStringList();
         THeadManager *defs = new THeadManager(msg);
         THeadManager *hdrs = new THeadManager(lines);
         GetAppData(msg,DefaultMessage);
         if (!(field=defs->GetDat("From:")).IsEmpty())
         {
            field = field.Trim();
            int pos = hdrs->IndexOf("From:");
            if (pos<0) lines->Insert(0,"From: "+field);
            else lines->Strings[pos] = "From: "+field;
         }
         if (!(field=defs->GetDat("Host: ")).IsEmpty())
         {
            field = field.Trim();
            int pos = hdrs->IndexOf("Host:");
            if (pos<0) lines->Insert(0,"Host: "+field);
            else lines->Strings[pos] = "Host: "+field;
         }
         win->Save();
         msg->Text = lines->Text;
         win->Editor->Clear();
         SetRedraw(win,0);
         lines->Text = msg->Text;
         Sleep(100);
         win->Editor->SelStart = 0;
         win->SetCursor();
         SetRedraw(win,1);
         delete hdrs;
         delete defs;
         delete msg;
      }
   }
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FileTemplatesItemClick(TObject *Sender)
{
   FileOpen(UserTemplates);
   CheckTemplateHostFrom();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FileOpenItemClick(TObject *Sender)
{
   FileOpen(UserMessages);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FileExitItemClick(TObject *Sender)
{
   Close();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FileSaveAsItemClick(TObject *Sender)
{
   if ((TextWin=dynamic_cast<TTextWin*>(ActiveMDIChild))!=NULL)
      TextWin->SaveAs();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FileSaveItemClick(TObject *Sender)
{
   if ((TextWin=dynamic_cast<TTextWin*>(ActiveMDIChild))!=NULL)
      TextWin->Save();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FilePrintItemClick(TObject *Sender)
{
   if ((TextWin=dynamic_cast<TTextWin*>(ActiveMDIChild))!=NULL)
      TextWin->Print();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FileNewNoteItemClick(TObject *Sender)
{
   bool locked = false;//LockWindowUpdate(GetDesktopWindow());
   try
   {
      NoteWin = new TNoteWin(this);
      NoteWin->Caption = newNote;
   }
   catch (...)
   {
      if (locked) LockWindowUpdate(0);
   }
   if (locked) LockWindowUpdate(0);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FileQueueItemClick(TObject *Sender)
{
   if ((MessageWin=dynamic_cast<TMessageWin*>(ActiveMDIChild))!=NULL)
      MessageWin->Queue(true);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::WMSendMail(TMessage& Msg)
{
   TMessageWin *msgwnd;
   if (AppDataExists(MailQueue))
   {
      if ((msgwnd=dynamic_cast<TMessageWin*>(ActiveMDIChild))!=NULL)
         msgwnd->Close();
      SendTimer->Enabled = true;
   }
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::SendTimerTimer(TObject *Sender)
{
   SendTimer->Enabled = false;
   SendMail();
   Perform(WM_UPDATEENABLE,HasMix()?1:0,NULL);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FileSendMessageItemClick(TObject *Sender)
{
   if ((MessageWin=dynamic_cast<TMessageWin*>(ActiveMDIChild))!=NULL)
      MessageWin->Send();
   else
   if (AppDataExists(MailQueue))
      SendMail();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FileSaveAsDefMessageItemClick(TObject *Sender)
{
   if ((MessageWin=dynamic_cast<TMessageWin*>(ActiveMDIChild))!=NULL)
      MessageWin->SaveDefMessage();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FileSaveAsDefArticleItemClick(TObject *Sender)
{
   if ((MessageWin=dynamic_cast<TMessageWin*>(ActiveMDIChild))!=NULL)
      MessageWin->SaveDefArticle();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::SetMenus(bool children)
{
   if (!children || !MDIChildCount)
   {
      bool queued = AppDataExists(MailQueue);
      if (queued) FileSendMessageItem->Caption = "Send Queued &Messages"; else
         FileSendMessageItem->Caption = "Send &Mail Now";
      FileSendMessageItem->Enabled = queued;

      FileQueueItem->Enabled = false;
      FileSaveItem->Enabled = false;
      FileSaveAsItem->Enabled = false;
      FileSaveAsTemplateItem->Enabled = false;
      FileSaveAsDefaultItem->Enabled = false;
      FilePrintItem->Enabled = false;
   }
   else
   if ((MessageWin=dynamic_cast<TMessageWin*>(ActiveMDIChild))!=NULL)
   {
      FileSendMessageItem->Enabled = true;
      FileQueueItem->Enabled = true;
      FileSaveAsTemplateItem->Enabled = true;
      FileSaveAsDefaultItem->Enabled = true;
      FileSaveItem->Enabled = true;
      FileSaveAsItem->Enabled = true;
      FilePrintItem->Enabled = true;
   }
   else
   if ((TextWin=dynamic_cast<TTextWin*>(ActiveMDIChild))!=NULL)
   {
      FileSendMessageItem->Enabled = false;
      FileQueueItem->Enabled = false;
      FileSaveAsTemplateItem->Enabled = false;
      FileSaveAsDefaultItem->Enabled = false;
      FileSaveItem->Enabled = true;
      FileSaveAsItem->Enabled = true;
      FilePrintItem->Enabled = true;
   }
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FileMenuClick(TObject *Sender)
{
   FileSMTPLogItem->Enabled = AppDataExists(SMTPLog);
   SessionLogsPOP3Item->Enabled = AppDataExists(POP3Log);
   FileOpenQueueItem->Enabled = AppDataExists(MailQueue);
   FileProblemsItem->Enabled = AppDataExists(ProblemQueue);
   FileTemplatesItem->Enabled = AppDataExists(UserTemplates);
   FileMiscMessagesItem->Enabled = AppDataExists(UserMessages);
   FileNotesItem->Enabled = AppDataExists(UserNotes);
   //FileNymsItem->Enabled = AppDataExists(UserNyms);
   FileSecureModeItem->Checked = !MasterPassword->IsEmpty();
   //if (!RegKeyExists(MixKey) || !hMixLib) FileSecureModeItem->Enabled = false;

   TRichEdit *Editor = NULL;
   MessageWin=NULL; TextWin=NULL; TextFileWin=NULL;
   if ((MessageWin=dynamic_cast<TMessageWin*>(ActiveMDIChild))!=NULL)
      Editor = MessageWin->Editor;
   else
   if ((TextFileWin=dynamic_cast<TTextFileWin*>(ActiveMDIChild))!=NULL)
      Editor = TextFileWin->Editor;
   else
   if ((TextWin=dynamic_cast<TTextWin*>(ActiveMDIChild))!=NULL)
      Editor = TextWin->Editor;
   if (!Editor) return;

   bool hasText = Editor->Lines->Count;
   FilePrintItem->Enabled = hasText;
   if (MessageWin)
   {
      FileSaveItem->Enabled = !MessageWin->Name().IsEmpty() && Editor->Modified;
      FileSaveAsItem->Enabled = hasText;
   }
   if (TextFileWin)
   {
      FileSaveItem->Enabled = Editor->Modified;
      FileSaveAsItem->Enabled = false;
   }
   if (TextWin)
   {
      FileSaveItem->Enabled = !TextWin->Name().IsEmpty() && Editor->Modified;
      FileSaveAsItem->Enabled = hasText;
   }

   int queued = (int)AppDataExists(MailQueue);
   if (!queued && !MessageWin)
      FileSendMessageItem->Caption = "Send &Mail Now";
   else
   if ( queued && !MessageWin)
      FileSendMessageItem->Caption = "Send Queued &Mail Now";
   else
   if (!queued &&  MessageWin)
      FileSendMessageItem->Caption = "Send &Message Now";
   else
   if ( queued &&  MessageWin)
      FileSendMessageItem->Caption = "Send &Mail Now";

   FileSendMessageItem->Enabled = queued || MessageWin;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::WindowItemClick(TObject *Sender)
{
   TMenuItem *item;
   if ((item=dynamic_cast<TMenuItem*>(Sender))!=NULL)
   {
      switch(item->Tag)
      {
         case 1: ArrangeIcons(); break;
         case 2: Cascade(); break;
         case 3: Tile(); break;
      }
   }
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::WindowMenuClick(TObject *Sender)
{
   WindowTileItem->Enabled = MDIChildCount>0;
   WindowCascadeItem->Enabled = MDIChildCount>0;
   WindowArrangeIconsItem->Enabled = MDIChildCount>0;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FileSaveAsTemplateItemClick(TObject *Sender)
{
   if ((MessageWin=dynamic_cast<TMessageWin*>(ActiveMDIChild))!=NULL)
      MessageWin->SaveAsTemplate();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::EditBtnClick(TObject *Sender)
{
   TOfficeButton *but;
   if ((TextWin=dynamic_cast<TTextWin*>(ActiveMDIChild))==NULL) return;
   if ((but=dynamic_cast<TOfficeButton*>(Sender))==NULL) return;
   switch (but->Tag)
   {
      case 1: EdCut->Click(); break;
      case 2: EdCopy->Click(); break;
      case 3: EdPaste->Click(); break;
   }
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::StatusBarResize(TObject *Sender)
{
   StatusBar->Panels->Items[0]->Width = Width-150;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::DisplayStatus(AnsiString &status, AnsiString &other)
{
   StatusBar->Panels->Items[0]->Text = status;
   StatusBar->Panels->Items[1]->Text = other;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::ClipboardMenuClick(TObject *Sender)
{
   bool enable = Clipboard()->HasFormat(CF_TEXT);
   ClipboardReplyItem->Enabled = enable;
   ClipboardFollowupItem->Enabled = enable;
   ClipboardReceiveItem->Enabled = enable;
/*
   ClipboardReplyItem->Enabled = false;
   ClipboardFollowupItem->Enabled = false;
   if (Clipboard()->HasFormat(CF_TEXT))
   {
      TMemo *m = new TMemo((void*)NULL);
      m->WordWrap = false;
      m->Visible = false;
      m->Parent = this;
      m->PasteFromClipboard();
      HeadMan->HereIs(m->Lines);
      if(HeadMan->Has("Newsgroups: ") || HeadMan->Has("Followup-To: "))
         ClipboardFollowupItem->Enabled = true;
      if (HeadMan->Has("Reply-To: ") || HeadMan->Has("From: "))
         ClipboardReplyItem->Enabled = true;
      delete m;
   }
*/
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Panel1Resize(TObject *Sender)
{
/*
   TControl *c;
   int i,tmp,left=0;
   for (i=0; i<Panel1->ControlCount; i++)
   {
      if ((c=dynamic_cast<TSpeedButton*>(Panel1->Controls[i]))==NULL) continue;
      tmp = c->Left+c->Width+68;
      if (tmp>left) left=tmp;
   }
   if (Panel1->Width>left)
      UpdateStatsBox->Left = Panel1->Width-68;
*/
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::HelpGettingStartedItemClick(TObject *Sender)
{
   AnsiString topics = "QuickQS";
   if (HasPgp()) topics += "\nQuickPGP";
   if (HasPop3()) topics += "\nQuickPOP3";
   OpenHelp(topics,"QuickStart","",this);
   //if (FileExists(AppDir+"doc\\QuickStart.rtf"))
   //   GetStarted(AppDir+"doc\\QuickStart.rtf");
   //else
   //if (FileExists(AppDir+"QuickStart.rtf"))
   //   GetStarted(AppDir+"QuickStart.rtf");
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::HelpQuickPop3ItemClick(TObject *Sender)
{
   OpenHelp("QuickPOP3","QuickStart POP3","",this);
   //if (FileExists(AppDir+"doc\\QuickStart.Pop3.rtf"))
   //   GetStarted(AppDir+"doc\\QuickStart.Pop3.rtf");
   //else
   //if (FileExists(AppDir+"QuickStart.Pop3.rtf"))
   //   GetStarted(AppDir+"QuickStart.Pop3.rtf");
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::HelpQuickPgpItemClick(TObject *Sender)
{
   OpenHelp("QuickPGP","QuickStart PGP","",this);
   //if (FileExists(AppDir+"doc\\QuickStart.Pgp.rtf"))
   //   GetStarted(AppDir+"doc\\QuickStart.Pgp.rtf");
   //else
   //if (FileExists(AppDir+"QuickStart.Pgp.rtf"))
   //   GetStarted(AppDir+"QuickStart.Pgp.rtf");
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FormMouseMove(TObject *Sender, TShiftState Shift,
  int X, int Y)
{
   rnd_mouse(WM_MOUSEMOVE,0,MAKELONG(X,Y));
   rnd_mouse_wipe(X,Y);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Preferences1Click(TObject *Sender)
{
   GetPreferences();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::MailboxOutboxItemClick(TObject *Sender)
{
   OpenMailbox("outbox");
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::MailboxInboxItemClick(TObject *Sender)
{
   OpenMailbox("inbox");
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::MailboxNewItemClick(TObject *Sender)
{
   AnsiString newname = CreateMailbox();
   if (!newname.IsEmpty()) OpenMailbox(newname);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::SessionLogsErrorCheckItemClick(TObject *Sender)
{
   TStringList *log = new TStringList();
   AnsiString title = OpenOpenDlg(log,ErrChkLog,!sorted);
   if (!title.IsEmpty())
      ReportWin = new TReportWin(log,title,Application->MainForm);
   delete log;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FileSMTPLogItemClick(TObject *Sender)
{
   TStringList *log = new TStringList();
   AnsiString title = OpenOpenDlg(log,SMTPLog,!sorted);
   if (!title.IsEmpty())
      ReportWin = new TReportWin(log,title,Application->MainForm);
   delete log;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FilePOP3LogItemClick(TObject *Sender)
{
   TStringList *log = new TStringList();
   AnsiString title = OpenOpenDlg(log,POP3Log,!sorted);
   if (!title.IsEmpty())
      ReportWin = new TReportWin(log,title,Application->MainForm);
   delete log;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::ViewChainLogClick(TObject *Sender)
{
   NoteWin = new TNoteWin(ChainLog,"Mixmaster Chain Log",this);
   NoteWin->Editor->SelStart = NoteWin->Editor->Lines->Text.Length();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::SavePOP3Log(WORD count, TStrings *log)
{
   TStringList *list = new TStringList();
   GetAppData(list,POP3Log);
   int max = GetAppIni(POP3LogMax,"").ToIntDef(16);
   while (max && list->Count>=max)
   {
      PutAppData(POP3Log+PS+list->Strings[0],NULL);
      list->Delete(0);
   }
   AnsiString logstr=SMTPPOP3logtime()+" "+log->Strings[log->Count-1];
   PutAppData(POP3Log+PS+logstr,log);
   delete list;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::CheckBtnClick(TObject *Sender)
{
   GetPop3Mail(this,Application->Handle,
      LastShiftState.Contains(ssCtrl)?true:false // in QSpop.dll
   );
}
void __fastcall TMainForm::FileCheckMailItemClick(TObject *Sender)
{
   GetPop3Mail(this,Application->Handle,LastShiftState.Contains(ssCtrl)?true:false); // in QSpop.dll
}
void __fastcall TMainForm::ToolsPopAccountsItemClick(TObject *Sender)
{
   GetPop3Mail(this,Application->Handle,true); // in QSpop.dll
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::ClipboardReceiveItemClick(TObject *Sender)
{
   TMemo *m = new TMemo((void*)NULL);
   m->Visible = false;
   m->WordWrap = false;
   m->Parent = this;
   m->PasteFromClipboard();
   THeadMan *head = new THeadMan(m->Lines);
   if ((head->Has("From: ") || head->Has("Reply-To:"))) //&& head->Has("To: "))
   {
      AnsiString idxstr = head->InIndexString(m->Lines);
      PutAppData(UserMailboxes+PS+"inbox"+PS+idxstr,m->Lines);
      OpenMessageIn("inbox",idxstr);
   }
   else
   {
      MsgBox(
         AnsiString("Clipboard does not appear to contain\n")+
         AnsiString("an email message!"),
         AnsiString("Can't Receive Clipboard"),
         MB_OK
      );
      TextWin = new TTextWin(this);
      TextWin->SetText("Clipboard Contents...",m->Lines);
   }
   delete head;
   delete m;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::HelpUpdateItemClick(TObject *Sender)
{
   bool cancel;
   RunProgramOnExit = RunUpdateExpress(this,cancel);
   if (cancel) Show(); else Close();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::HelpMenuClick(TObject *Sender)
{
   AnsiString DocDir=DirectoryExists(AppDir+"doc")?AppDir+"doc\\":AppDir;
   HelpGettingStartedItem->Enabled = FileExists(DocDir+QuickStart);
   //HelpQuickPop3Item->Visible = HasPop3() && FileExists(DocDir+QuickStartPop3);
   //HelpQuickPgpItem->Visible = HasPgp() && FileExists(DocDir+QuickStartPgp);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::ToolsOptionsItemsClick(TObject *Sender)
{
   GetPreferences();
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::InBoxBtnClick(TObject *Sender)
{
   OpenMailbox("inbox");
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::OutBoxBtnClick(TObject *Sender)
{
   OpenMailbox("outbox");
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::OptionsBtnClick(TObject *Sender)
{
   GetPreferences();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::ViewClearItemClick(TObject *Sender)
{
   TStringList *files = new TStringList();
   files->Text = ClearChosenDocuments(this);
   for (int i=0; i<files->Count; i++)
      DeleteFile(files->Strings[i].c_str());
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::SetKeyringMenu()
{
   for (int i=0; i<KeyringMenu->Count; i++)
      KeyringMenu->Items[i]->Enabled = false;
   TForm *Active = ActiveMDIChild;

   if (dynamic_cast<TPgpKeyWin*>(Active)!=NULL)
   {
      //KeyringMenu->Visible = true;
      KeyringNewKeyItem->Visible = true;
      KeyringDisableItem->Visible = true;
      N11->Visible = true;
      KeyringClearItem->Visible = false;
      KeyringSaveItem->Visible = true;    //NOPGP6SAVE ? true:false;
      KeyringSaveMenu->Visible = false;   //NOPGP6SAVE ? false:true;
      KeyringRefreshItem->Visible = false;
      KeyringRefreshMenu->Visible = true;
      KeyringImportItem->Visible = true;
      N15->Visible = true;
      KeyringDeleteItem->Visible = true;
      N16->Visible = false;
      KeyringConfigItem->Visible = false;
      KeyringOpenItem->Visible = false;
      return;
   }

   if (dynamic_cast<TMixProbWin*>(Active)!=NULL)
   {
      //KeyringMenu->Visible = true;
      KeyringNewKeyItem->Visible = false;
      KeyringDisableItem->Visible = true;
      N11->Visible = true;
      KeyringClearItem->Visible = true;
      KeyringSaveItem->Visible = false;
      KeyringSaveMenu->Visible = false;
      KeyringRefreshItem->Visible = false;
      KeyringRefreshMenu->Visible = false;
      KeyringImportItem->Visible = false;
      N15->Visible = false;
      KeyringDeleteItem->Visible = true;
      N16->Visible = false;
      KeyringConfigItem->Visible = true;
      KeyringOpenItem->Visible = false;
      return;
   }

   if (dynamic_cast<TMixKeyWin*>(Active)!=NULL)
   {
      //KeyringMenu->Visible = true;
      KeyringNewKeyItem->Visible = false;
      KeyringDisableItem->Visible = true;
      N11->Visible = true;
      KeyringClearItem->Visible = true;
      KeyringSaveItem->Visible = true;
      KeyringSaveMenu->Visible = false;
      KeyringRefreshItem->Visible = true;
      KeyringRefreshMenu->Visible = false;
      KeyringImportItem->Visible = true;
      N15->Visible = true;
      KeyringDeleteItem->Visible = true;
      N16->Visible = true;
      KeyringConfigItem->Visible = true;
      KeyringOpenItem->Visible = true;
      return;
   }

   TCpkKeyWin *ckeywin = dynamic_cast<TCpkKeyWin*>(Active);
   if (ckeywin)
   {
      //KeyringMenu->Visible = true;
      KeyringNewKeyItem->Visible = false;
      KeyringDisableItem->Visible = true;
      N11->Visible = false;
      KeyringClearItem->Visible = false;
      KeyringSaveItem->Visible = true;
      KeyringSaveMenu->Visible = false;
      KeyringRefreshItem->Visible = true;
      KeyringRefreshMenu->Visible = false;
      KeyringImportItem->Visible = true;
      N15->Visible = true;
      KeyringDeleteItem->Visible = true;
      N16->Visible = true;
      KeyringConfigItem->Visible = true;
      KeyringOpenItem->Visible = true;
      return;
   }

   //KeyringMenu->Visible = false;
   KeyringNewKeyItem->Visible = true;
   KeyringDisableItem->Visible = true;
   N11->Visible = true;
   KeyringClearItem->Visible = true;
   KeyringSaveItem->Visible = true;
   KeyringSaveMenu->Visible = false;
   KeyringRefreshItem->Visible = true;
   KeyringRefreshMenu->Visible = false;
   KeyringImportItem->Visible = true;
   N15->Visible = true;
   KeyringDeleteItem->Visible = true;
   N16->Visible = false;
   KeyringConfigItem->Visible = false;
   KeyringOpenItem->Visible = false;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::SetEditMenu()
{
   TForm *active = ActiveMDIChild;
   // reset all items to invisible and disabled
   for (int i=0; i<EdMenu->Count; i++)
      EdMenu->Items[i]->Enabled = false;
   // set items based on mdichild type
   if (!MDIChildCount)
   {
      EdUndo->Visible = true;
      EdUndoBrk->Visible = true;
      EdCut->Visible = true;
      EdCopy->Visible = true;
      EdPaste->Visible = true;
      EdPasteBrk->Visible = true;
      EdSelectAll->Visible = true;
      EdSelectBrk->Visible = false;
      EdInsertFilename->Visible = false;
      EdInsertText->Visible = false;
      EdInsertBrk->Visible = false;
      EdWordWrapMessage->Visible = false;
      EdWordWrapNote->Visible = false;
      EdWordWrap->Visible = false;
      return;
   }
   if (dynamic_cast<TMailboxWin*>(active)!=NULL)
   {
      EdUndo->Visible = true;
      EdUndoBrk->Visible = true;
      EdCut->Visible = true;
      EdCopy->Visible = true;
      EdPaste->Visible = true;
      EdPasteBrk->Visible = true;
      EdSelectAll->Visible = true;
      EdSelectBrk->Visible = false;
      EdInsertFilename->Visible = false;
      EdInsertText->Visible = false;
      EdInsertBrk->Visible = false;
      EdWordWrapMessage->Visible = false;
      EdWordWrapNote->Visible = false;
      EdWordWrap->Visible = false;
      return;
   }
   if (dynamic_cast<TTextWin*>(active)!=NULL)
   {
      EdUndo->Visible = true;
      EdUndoBrk->Visible = true;
      EdCut->Visible = true;
      EdCopy->Visible = true;
      EdPaste->Visible = true;
      EdPasteQuote->Visible = true;
      EdPasteBrk->Visible = true;
      EdSelectAll->Visible = true;
      EdSelectBrk->Visible = true;
      EdInsertFilename->Visible = true;
      EdInsertText->Visible = true;
      EdInsertBrk->Visible = false;
      EdWordWrapMessage->Visible = false;
      EdWordWrapNote->Visible = false;
      EdWordWrap->Visible = false;
   }
   if (dynamic_cast<TMixKeyWin*>(active)!=NULL ||
       dynamic_cast<TCpkKeyWin*>(active)!=NULL ||
       dynamic_cast<TPgpKeyWin*>(active)!=NULL)
   {
      EdUndo->Visible = true;
      EdUndoBrk->Visible = true;
      EdCut->Visible = true;
      EdCopy->Visible = true;
      EdPaste->Visible = true;
      EdPasteBrk->Visible = true;
      EdSelectAll->Visible = true;
      EdSelectBrk->Visible = false;
      EdInsertFilename->Visible = false;
      EdInsertText->Visible = false;
      EdInsertBrk->Visible = false;
      EdWordWrapMessage->Visible = false;
      EdWordWrapNote->Visible = false;
      EdWordWrap->Visible = false;
   }
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::EdMenuActionClick(TObject *Sender)
{
   TTextWin *tw;
   //TAsciiDBWin *db;
   TMailboxWin *mbw;
   TForm *active = ActiveMDIChild;
   mbw = dynamic_cast<TMailboxWin*>(active);
   if (!mbw) tw = dynamic_cast<TTextWin*>(active);
   //if (!tw ) db = dynamic_cast<TAsciiDBWin*>(active);
   TMenuItem *item = dynamic_cast<TMenuItem*>(Sender);
   if (item)
   {
      if (item->Caption=="&Undo")
      {
         if (tw) tw->Editor->Perform(WM_UNDO,0,NULL);
         //if (db) db->Editor->Perform(WM_UNDO,0,NULL);
      }
      else
      if (item->Caption=="Cu&t")
      {
         if (tw) tw->CutToClipboard();
         /*if (db)
         {
            db->Editor->CutToClipboard();
            db->SaveBtn->Enabled=true;
         }*/
      }
      else
      if (item->Caption=="&Copy")
      {
         if (tw) tw->Editor->CopyToClipboard();
         //if (db) db->Editor->CopyToClipboard();
      }
      else
      if (item->Caption=="&Paste")
      {
         if (tw) tw->PasteFromClipboard();
         /*if (db)
         {
            db->Editor->PasteFromClipboard();
            db->SaveBtn->Enabled=true;
         }*/
      }
      else
      if (item->Caption=="Paste As &Quotation")
      {
         if (tw) tw->PasteFromClipboardQuote();
      }
      else
      if (item->Caption=="Select &All")
      {
         if (mbw) mbw->MailGrid->SelectAll(); else
         if (tw) tw->Editor->SelectAll(); //else
         //if (db) db->Editor->SelectAll();
      }
      else
      if (item->Caption=="Insert &Text From File...")
      {
         if (tw) tw->InsertFileText();
      }
      else
      if (item->Caption=="Insert &Filename...")
      {
         if (tw) tw->InsertFileName();
      }
      else
      if (item->Caption=="&Word Wrap")
      {
         GetPreferences("Editor");
      }
   }
}
//---------------------------------------------------------------------------
// function above and below need mod when adding new mdichild types
//---------------------------------------------------------------------------
void __fastcall TMainForm::EdMenuClick(TObject *Sender)
{
   TTextWin *tw;
   TForm *active = ActiveMDIChild;
   if (!active)
   {
      EdWordWrap->Enabled = true;
      EdWordWrap->Checked = GetAppIni("WordWrap","").ToIntDef(1)>0;
      return;
   }
   if (dynamic_cast<TMailboxWin*>(active)!=NULL)
   {
      EdSelectAll->Enabled = true;
      EdWordWrap->Enabled = true;
      EdWordWrap->Checked = GetAppIni("WordWrap","").ToIntDef(1)>0;
      return;
   }
   if ((tw=dynamic_cast<TTextWin*>(active))!=NULL)
   {
      bool hasSel = tw->Editor->SelLength > 0;
      EdUndo->Enabled = tw->Editor->Modified;
      EdCut->Enabled = hasSel;
      EdCopy->Enabled = hasSel;
      EdPaste->Enabled = Clipboard()->HasFormat(CF_TEXT);
      EdPasteQuote->Enabled = EdPaste->Enabled;
      EdSelectAll->Enabled = tw->Editor->Lines->Count > 0;
      EdInsertFilename->Enabled = true;
      EdInsertText->Enabled = true;
      EdWordWrap->Enabled = true;
      EdWordWrap->Checked = GetAppIni("WordWrap","").ToIntDef(1)>0;
   }
   if (dynamic_cast<TMixKeyWin*>(active)!=NULL ||
       dynamic_cast<TPgpKeyWin*>(active)!=NULL)
   {
      EdUndo->Enabled = false;
      EdCut->Enabled = false;
      EdCopy->Enabled = false;
      EdPaste->Enabled = false;
      EdSelectAll->Enabled = false;
   }
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::ToolsMixManItemClick(TObject *Sender)
{
   if (!FileExists(MixDir()+"mlist.txt") && !FileExists(MixDir()+"pubring.mix") &&
        GetFirstRemailerDocs(this))
   {
      GetRemailerStats(this);
      return;
   }
   // if the keyring is open MixKeyWin points to it
   if (!MixKeyWin) MixKeyWin = new TMixKeyWin(this,mringtxt,mtyp2txt);
   else MixKeyWin->BringToFront();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::ToolsCpunkManItemClick(TObject *Sender)
{
   if (!CpkKeyWin) CpkKeyWin = new TCpkKeyWin(this,cringtxt,ctyp1txt,clisttxt);
   else CpkKeyWin->BringToFront();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::ToolsPgpKeyItemClick(TObject *Sender)
{
   if (!PgpKeyWin) PgpKeyWin = new TPgpKeyWin(this,PgpPublicRing(),PgpSecretRing());
   else PgpKeyWin->BringToFront();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::KeyringMenuClick(TObject *Sender)
{
   KeyringMenuDat menudat;
   TForm *Active = ActiveMDIChild;

   if (dynamic_cast<TPgpKeyWin*>(Active)!=NULL)
   {
      if (Active->Perform(WM_GETKEYRINGMENUDAT,0,(LPARAM)&menudat))
      {
         bool hasSel=menudat.HasSel;
         KeyringNewKeyItem->Enabled = true;
         KeyringDisableItem->Enabled = hasSel && !menudat.IsPair;
         KeyringDisableItem->Caption =(hasSel &&  menudat.CurEnabled ? "Disable this &key":"Enable this &key");
         KeyringSaveItem->Enabled = menudat.Modified;
         KeyringSaveMenu->Enabled = true; //menudat.Modified;
         KeyringSaveDiskItem->Enabled = menudat.Modified;
         KeyringRefreshMenu->Enabled = true;
         KeyringRefreshDiskItem->Enabled = menudat.Modified;
         KeyringImportItem->Enabled = true;
         KeyringDeleteItem->Enabled = hasSel;
      }
      return;
   }

   if (dynamic_cast<TMixProbWin*>(Active)!=NULL)
   {
      if (Active->Perform(WM_GETKEYRINGMENUDAT,0,(LPARAM)&menudat))
      {
         bool hasSel=menudat.HasSel;
         KeyringDisableItem->Enabled = hasSel;
         KeyringClearItem->Enabled = menudat.Count>0;
         KeyringDeleteItem->Enabled = hasSel;
         KeyringConfigItem->Enabled = hasSel;
      }
      return;
   }

   if (dynamic_cast<TMixKeyWin*>(Active)!=NULL)
   {
      if (Active->Perform(WM_GETKEYRINGMENUDAT,0,(LPARAM)&menudat))
      {
         bool hasSel=menudat.HasSel;
         KeyringDisableItem->Enabled = hasSel;
         KeyringDisableItem->Caption = hasSel && menudat.CurEnabled ? "Disable this &key":"Enable this &key";
         KeyringClearItem->Enabled = menudat.Count>0;
         KeyringSaveItem->Enabled = menudat.Modified;
         KeyringRefreshItem->Enabled = menudat.RefreshEnabled;
         KeyringImportItem->Enabled = true;
         KeyringDeleteItem->Enabled = hasSel;
         KeyringConfigItem->Enabled = hasSel;
         KeyringOpenItem->Enabled = true;
      }
      return;
   }

   TCpkKeyWin *ckeywin = dynamic_cast<TCpkKeyWin*>(Active);
   if (ckeywin)
   {
      bool hasSel=ckeywin->curkey>=0;
      KeyringSaveItem->Enabled = ckeywin->modified || ckeywin->typlistmod;
      KeyringRefreshItem->Enabled = ckeywin->RefreshBtn->Enabled;
      KeyringImportItem->Enabled = true;
      KeyringDisableItem->Enabled = hasSel;
      KeyringDisableItem->Caption =
         (hasSel && ckeywin->EnabledKey(ckeywin->curkey) ? "Disable this &key":"Enable this &key");
      KeyringDeleteItem->Enabled = hasSel;
      KeyringConfigItem->Enabled = hasSel;
      KeyringOpenItem->Enabled = true;
      return;
   }

   KeyringNewKeyItem->Enabled = false;
   KeyringDisableItem->Enabled = false;
   KeyringClearItem->Enabled = false;
   KeyringSaveItem->Enabled = false;
   KeyringRefreshItem->Enabled = false;
   KeyringImportItem->Enabled = false;
   KeyringDeleteItem->Enabled = false;
   KeyringConfigItem->Enabled = false;
   KeyringOpenItem->Enabled = false;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::KeyringOpenItemClick(TObject *Sender)
{
   for (int i=0; i<KeyringOpenItem->Count; i++)
      KeyringOpenItem->Items[i]->Visible = false;
   TForm *Active = ActiveMDIChild;

   if (dynamic_cast<TMixProbWin*>(Active)!=NULL)
   {
      KeyringOpenProblemmixItem->Visible = true;
      KeyringOpenProblemmixItem->Enabled = FileExists(mprobtxt);
      return;
   }

   if (dynamic_cast<TMixKeyWin*>(Active)!=NULL)
   {
      KeyringOpenPubringmixItem->Visible = true;
      KeyringOpenPubringmixItem->Enabled = FileExists(mringtxt);
      KeyringOpenType2Item->Visible = true;
      KeyringOpenType2Item->Enabled = FileExists(mtyp2txt);
      KeyringOpenMlistItem->Visible = true;
      KeyringOpenMlistItem->Enabled = FileExists(mlisttxt);
      return;
   }

   if (dynamic_cast<TCpkKeyWin*>(Active)!=NULL)
   {
      KeyringOpenPubringascItem->Visible = true;
      KeyringOpenPubringascItem->Enabled = FileExists(cringtxt);
      KeyringOpenType1Item->Visible = DocumentErrorChecking();
      KeyringOpenType1Item->Enabled = FileExists(ctyp1txt);
      KeyringOpenRlistItem->Visible = true;
      KeyringOpenRlistItem->Enabled = FileExists(clisttxt);
      return;
   }
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::KeyringSaveItemClick(TObject *Sender)
{
   ActiveMDIChild->Perform(WM_SAVEBTNCLICKED,0,0);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::KeyringRefreshItemClick(TObject *Sender)
{
   ActiveMDIChild->Perform(WM_REFRESHBTNCLICKED,0,0);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::KeyringImportItemClick(TObject *Sender)
{
   ActiveMDIChild->Perform(WM_IMPORTBTNCLICKED,0,0);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::KeyringDisableItemClick(TObject *Sender)
{
   ActiveMDIChild->Perform(WM_DISABLEBTNCLICKED,0,0);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::KeyringDeleteItemClick(TObject *Sender)
{
   ActiveMDIChild->Perform(WM_DELETEBTNCLICKED,0,0);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::KeyringConfigItemClick(TObject *Sender)
{
   ActiveMDIChild->Perform(WM_KEYRINGCONFIGITEMCLICKED,0,0);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::KeyringOpenDocItemClick(TObject *Sender)
{
   ActiveMDIChild->Perform(WM_KEYRINGOPENDOCITEMCLICKED,0,0);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Applicationdirectory1Click(TObject *Sender)
{
   Exec95("explorer "+AppDir);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::OpenMixDirectoryItemClick(TObject *Sender)
{
   Exec95("explorer "+MixDir());
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::KeyringClearItemClick(TObject *Sender)
{
   ActiveMDIChild->Perform(WM_KEYRINGCLEARITEMCLICKED,0,0);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::KeyringNewKeyItemClick(TObject *Sender)
{
   ActiveMDIChild->Perform(WM_KEYRINGNEWKEYITEMCLICKED,0,0);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Contents1Click(TObject *Sender)
{
   BrowseHelpTopics(this);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::KeyringRefreshPGP6ItemClick(TObject *Sender)
{
   TPgpKeyWin *pgpwin = dynamic_cast<TPgpKeyWin*>(ActiveMDIChild);
   if (pgpwin) pgpwin->RefreshFromPGP6Item->Click();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::KeyringRefreshDiskItemClick(TObject *Sender)
{
   TPgpKeyWin *pgpwin = dynamic_cast<TPgpKeyWin*>(ActiveMDIChild);
   if (pgpwin) pgpwin->RefreshFromDiskItemClick(NULL);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::KeyringSaveDiskItemClick(TObject *Sender)
{
   TPgpKeyWin *pgpwin = dynamic_cast<TPgpKeyWin*>(ActiveMDIChild);
   if (pgpwin) pgpwin->SaveBtn->Click();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::KeyringSavePGP6ItemClick(TObject *Sender)
{
   //TPgpKeyWin *pgpwin = dynamic_cast<TPgpKeyWin*>(ActiveMDIChild);
   //if (pgpwin) pgpwin->CheckSaveToPGP6();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FormResize(TObject *Sender)
{
   HelpBtn->Left = Width-(HelpBtn->Width+10);
}
//---------------------------------------------------------------------------
